/*******************************************************************************
 * Copyright (c) 2011, 2018 fortiss GmbH.
 * 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
 * which is available at https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0
 *******************************************************************************/
package org.fortiss.tooling.kernel.ui.introspection.details;

import java.util.HashMap;

import org.fortiss.tooling.kernel.introspection.IIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.IIntrospectionItem;
import org.fortiss.tooling.kernel.introspection.items.ConnectionCompositorServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.items.ConstraintCheckerServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.items.EclipseResourceStorageServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.items.ElementCompositorServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.items.LibraryServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.items.MigrationServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.items.PersistencyServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.items.PrototypeServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.items.TransformationServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.ui.introspection.details.handler.AllocationEditPartFactoryServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.introspection.details.handler.ConnectionCompositorServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.introspection.details.handler.ConstraintCheckerServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.introspection.details.handler.ContextMenuServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.introspection.details.handler.EclipseResourceStorageProviderIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.introspection.details.handler.EditPartFactoryServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.introspection.details.handler.ElementCompositorServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.introspection.details.handler.LibraryServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.introspection.details.handler.MigrationServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.introspection.details.handler.ModelEditorBindingServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.introspection.details.handler.ModelElementHandlerServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.introspection.details.handler.PersistencyServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.introspection.details.handler.PrototypeServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.introspection.details.handler.TransformationServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.introspection.items.AllocationEditPartFactoryServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.ui.introspection.items.ContextMenuServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.ui.introspection.items.EditPartFactoryServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.ui.introspection.items.ModelEditorBindingServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.ui.introspection.items.ModelElementHandlerServiceIntrospectionDetailsItem;

/** Registry class to provide the details GUI for a given {@link IIntrospectionItem}. */
public final class KISSDetailsUIRegistry {

	/** Returns the singleton instance of the service. */
	private static final KISSDetailsUIRegistry INSTANCE = new KISSDetailsUIRegistry();

	/** Returns the singleton instance. */
	public static KISSDetailsUIRegistry getInstance() {
		return INSTANCE;
	}

	/** The registry. */
	private final HashMap<Class<? extends IIntrospectionDetailsItem>, DetailsUIHandlerBase> registry =
			new HashMap<Class<? extends IIntrospectionDetailsItem>, DetailsUIHandlerBase>();

	/** Constructor. */
	public KISSDetailsUIRegistry() {
		registerHandler(AllocationEditPartFactoryServiceIntrospectionDetailsItem.class,
				new AllocationEditPartFactoryServiceIntrospectionDetailsUIHandler());
		registerHandler(ConnectionCompositorServiceIntrospectionDetailsItem.class,
				new ConnectionCompositorServiceIntrospectionDetailsUIHandler());
		registerHandler(ConstraintCheckerServiceIntrospectionDetailsItem.class,
				new ConstraintCheckerServiceIntrospectionDetailsUIHandler());
		registerHandler(ContextMenuServiceIntrospectionDetailsItem.class,
				new ContextMenuServiceIntrospectionDetailsUIHandler());
		registerHandler(EclipseResourceStorageServiceIntrospectionDetailsItem.class,
				new EclipseResourceStorageProviderIntrospectionDetailsUIHandler());
		registerHandler(EditPartFactoryServiceIntrospectionDetailsItem.class,
				new EditPartFactoryServiceIntrospectionDetailsUIHandler());
		registerHandler(ElementCompositorServiceIntrospectionDetailsItem.class,
				new ElementCompositorServiceIntrospectionDetailsUIHandler());
		registerHandler(LibraryServiceIntrospectionDetailsItem.class,
				new LibraryServiceIntrospectionDetailsUIHandler());
		registerHandler(MigrationServiceIntrospectionDetailsItem.class,
				new MigrationServiceIntrospectionDetailsUIHandler());
		registerHandler(ModelEditorBindingServiceIntrospectionDetailsItem.class,
				new ModelEditorBindingServiceIntrospectionDetailsUIHandler());
		registerHandler(ModelElementHandlerServiceIntrospectionDetailsItem.class,
				new ModelElementHandlerServiceIntrospectionDetailsUIHandler());
		registerHandler(PrototypeServiceIntrospectionDetailsItem.class,
				new PrototypeServiceIntrospectionDetailsUIHandler());
		registerHandler(PersistencyServiceIntrospectionDetailsItem.class,
				new PersistencyServiceIntrospectionDetailsUIHandler());
		registerHandler(TransformationServiceIntrospectionDetailsItem.class,
				new TransformationServiceIntrospectionDetailsUIHandler());
	}

	/** Registers the given composite */
	public void registerHandler(Class<? extends IIntrospectionDetailsItem> clazz,
			DetailsUIHandlerBase detailsHandler) {
		registry.put(clazz, detailsHandler);
	}

	/** Returns the composite factory for the given class. */
	public DetailsUIHandlerBase getHandler(Class<? extends IIntrospectionDetailsItem> clazz) {
		return registry.get(clazz);
	}
}
